#!/usr/bin/env python3
"""
Find import files from the x3 import archive folder
and stick them together for a single reimport.

Useful when stock levels prevented multiple shipments from
importing in.
"""
import shutil
import pathlib
import pprint

THIS_DIRECTORY = pathlib.Path(__file__).parent
X3_IMPORT_ARCHIVE = THIS_DIRECTORY / "to_import_SOH" / "archive"
REIMPORT_DIRECTORY = THIS_DIRECTORY / "shipment_reimport"


LIST_OF_UNIMPORTED_ORDERS = [
'514113',
'514092',
'513984',
'514110',
'514055',
'514141',
'514062',
'514051',
'514142',
'514095',
'513897',
'513918',
'513896',
'513884',
'513927'
]


def main():
    #run through all files in list and combine them into a single import
    with open(REIMPORT_DIRECTORY / 'ZSHPORD.dat', 'w',encoding="utf-8", newline="\n") as combined_import_file:
        files = get_files()
        for file in files:
            with file.open(
                    "r", encoding="utf-8", newline="\n"
                ) as individual_import_file:
                for line in individual_import_file:
                    combined_import_file.write(line)
    

#search the archive directory for the files, write their contents to a single file
def get_files():
    file_list = []
    for missing_order in LIST_OF_UNIMPORTED_ORDERS:
        for file in X3_IMPORT_ARCHIVE.iterdir():
            order_number = file.name[8:14]
            if missing_order == order_number:
                pprint.pprint(f'found {missing_order}')
                file_list.append(file)
    return file_list


if __name__ == "__main__":
    main()        